#include "entities.h"
#include "frames.h"
#include "save.h"

#include <stdlib.h>

//---
//	Skill management.
//	Should have done this using structures, but weird values initially used
//	for skills (thought to be used in an OR-combination) prevented this.
//	Now it's too late, I lack the time necessary to do this edit.
//---

/*
	skill_type()
	Returns the type of a given skill.
*/
enum SkillType skill_type(enum Skill skill)
{
	switch(skill)
	{
	case Skill_Fire:
	case Skill_Thunder:
	case Skill_Special:
		return SkillType_Damage;

	case Skill_Cure:
		return SkillType_Heal;

	case Skill_None:
		return SkillType_None;
	}

	return SkillType_None;
}

/*
	skill_power()
	Returns the power level of the skill (base amount of damage dealt or
	hp healed, for instance).
*/
int skill_power(enum Skill skill)
{
	switch(skill)
	{
	case Skill_Fire:	return 40;
	case Skill_Thunder:	return 25;
	case Skill_Special:	return save()->entity.atk * 3;

	case Skill_Cure:	return save()->entity.max_hp / 2;

	case Skill_None:	return 0;
	}

	return 0;
}

/*
	skill_cost()
	Returns the MP cost of a skill.
*/
int skill_cost(enum Skill skill)
{
	switch(skill)
	{
	case Skill_Fire:	return 4;
	case Skill_Thunder:	return 3;
	case Skill_Special:	return 3;

	case Skill_Cure:	return 5;

	case Skill_None:	return 0;
	}

	return 0;
}

/*
	skill_visual()
	Returns the visual effect enumeration item associated to a skill.
*/
enum Visual skill_visual(enum Skill skill)
{
	switch(skill)
	{
	case Skill_Thunder:	return Visual_Thunder;
	case Skill_Fire:	return Visual_Fire;
	case Skill_Special:	return Visual_Special;

	case Skill_Cure:	return Visual_Heal;

	case Skill_None:	return Visual_Damage;
	}

	return Visual_Damage;
}



//---
//	Enemy base statistics.
//---

static struct MemoryEntity enemies[] = {
	{  "SLIME",     15,  0,    8,  3, 1, 
		{ Skill_None, Skill_None, Skill_None } },
	{  "SPIDER",    25,  0,   10,  5, 6,
		{ Skill_None, Skill_None, Skill_None } },
	{  "RAT",       10,  4,   13,  8, 4,
		{ Skill_Fire, Skill_None, Skill_None } },
	{  "SOLDIER",   40,  3,   16,  7, 5,
		{ Skill_Thunder, Skill_None, Skill_None } },
	{  "ROBOT1",     8,  0,    6,  2, 1,
		{ Skill_None, Skill_None, Skill_None } },
	{  "ROBOT2",    20,  3,    8,  4, 1,
		{ Skill_Thunder, Skill_None, Skill_None } },
	{  "GUARD",     65,  4,   20, 14, 6,
		{ Skill_Fire, Skill_None, Skill_None } },
	{  "LION",     240,  6,   30, 10, 2,
		{ Skill_Thunder, Skill_None, Skill_None } },
};

/*
	convert()
	Converts a memory entity to a battle entity. Only copies fields that
	are present in both structures. Also applies a level variation.
*/
static struct BattleEntity convert(struct MemoryEntity *m, int level)
{
	struct BattleEntity e;

	e.name		= m->name;
	e.level		= level;

	e.max_hp	= m->max_hp	* level;
	e.max_mp	= m->max_mp	* level;
	e.atk		= m->atk	* level;
	e.def		= m->def	* level;
	e.spe		= m->spe	* level;

	e.skills[0]	= m->skills[0];
	e.skills[1]	= m->skills[1];
	e.skills[2]	= m->skills[2];

	return e;
}

/*
	entity()
	Returns fresh editable data for a given entity.
*/
struct BattleEntity entity(enum Character character, int level)
{
	struct BattleEntity e = {
		NULL, NULL, 0,
		0, 0, 0,
		0, 0, 0,
		0, 0, 0,
		Stance_Fight, { 0, 0, 0 }
	};

	if(character == Character_None) return e;
	if(character == Character_Player)
	{
		e = convert(&(save()->entity), 1);
		e.sprite = NULL;
	}
	else
	{
		e = convert(&enemies[character], level);
		e.sprite = frames_enemy(character);
	}

	e.hp = e.disp_hp = e.max_hp;
	e.mp = e.disp_mp = e.max_mp;
	e.stance = Stance_Fight;

	return e;
}

/*
	entity_hit()
	Lets the fighter attack the target.
	"It hurts!"  -- a giant spider
*/
void entity_hit(BE *fighter, BE *target)
{
	// Between 80 and 130%.
	int random = 80 + rand() % 51;
	int damage = (fighter->atk - target->def) * random / 100;

	if(damage < 0) damage = 0;
	if(target->stance == Stance_Guard) damage /= 2;

	target->hp -= damage;
	if(target->hp < 0) target->hp = 0;
}

/*
	entity_skill()
	Uses a skill on a target.
*/
void entity_skill(BE *target, enum Skill skill)
{
	int random = 80 + rand() % 51;
	int amount = skill_power(skill) * random / 100;

	switch(skill_type(skill))
	{
	case SkillType_Damage:
		if(target->stance == Stance_Guard) amount /= 2;
		target->hp -= amount;
		if(target->hp < 0) target->hp = 0;
		break;

	case SkillType_Heal:
		target->hp += amount;
		if(target->hp > target->max_hp) target->hp = target->max_hp;
		break;

	case SkillType_None:
	default:
		break;
	}
}
