#ifndef	_ENTITIES_H
#define	_ENTITIES_H

//---
//	Skill management.
//---

/*
	enum Skill
	A skill list.
*/
enum Skill
{
	Skill_None	= -1,
	Skill_Fire	= 0,
	Skill_Thunder	= 1,
	Skill_Cure	= 2,
	Skill_Special	= 3,
};

enum SkillType
{
	SkillType_None		= 0,
	SkillType_Damage	= 1,
	SkillType_Heal		= 2,
};

/*
	skill_type()
	Returns the type of a given skill.
*/
enum SkillType skill_type(enum Skill skill);

/*
	skill_power()
	Returns the power level of the skill (base amount of damage dealt or
	hp healed, for instance).
*/
int skill_power(enum Skill skill);

/*
	skill_cost()
	Returns the MP cost of a skill.
*/
int skill_cost(enum Skill skill);

/*
	skill_visual()
	Returns the visual effect enumeration item associated to a skill.
*/
enum Visual skill_visual(enum Skill skill);



//---
//	Entity management.
//---

/*
	enum Stance
	Different kind of stances with different king of effects.
*/
enum Stance
{
	Stance_Fight,
	Stance_Guard,
};

/*
	struct MemoryEntity
	Entities in memory. Give information about stats.
*/
struct MemoryEntity
{
	const char *name;
	int max_hp, max_mp;
	int atk, def, spe;
	enum Skill skills[3];
};

/*
	struct BattleEntity
	Entities in battle. Additional informations include current hp and mp,
	hp and mp currently displayed, dead indicators and the like. Spells
	are broken down into three values to make access easier.
*/
struct BattleEntity
{
	struct Sprite *sprite;
	const char *name;
	int level;

	int max_hp, hp, disp_hp;
	int max_mp, mp, disp_mp;

	int atk, def, spe;
	enum Stance stance;
	enum Skill skills[3];
};
typedef struct BattleEntity BE;

/*
	enum Character
	Anything that fights, again.
*/
enum Character
{
	Character_None		= -2,
	Character_Player	= -1,

	Character_Slime		= 0,
	Character_Spider	= 1,
	Character_Rat		= 2,
	Character_Soldier	= 3,

	Character_Robot1	= 4,
	Character_Robot2	= 5,

	Character_Guard		= 6,
	Character_Lion		= 7,

	Character_Max		= 7,
};

#include "frames.h"

/*
	entity()
	Returns an instance of an entity, along with battle-suitable data.
*/
struct BattleEntity entity(enum Character entity, int level);

/*
	entity_hit()
	Lets the fighter attack the target.
	"It hurts!"  -- a giant spider
*/
void entity_hit(BE *fighter, BE *target);

/*
	entity_skill()
	Uses a skill on a target.
*/
void entity_skill(BE *target, enum Skill skill);

#endif	// _ENTITIES_H
