#include "frames.h"

//---
//	Game sprites.
//---

extern Image
	res_sprites_start,
	res_enemies_start,
	res_visuals_start;

// Player sprites.
static struct Sprite sprites[] = {
	// Idle frames.
	{  &res_sprites_start,   7,  9,    0,   0, 20, 18  },
	{  &res_sprites_start,   7,  8,    0,  19, 20, 17  },
	// Sword fight.
	{  &res_sprites_start,   7,  9,    0,  37, 18, 19  },
	{  &res_sprites_start,  18,  9,    0,  57, 28, 18  },
	// This is as far as you can get...
	{  &res_sprites_start,   7,  7,    0,  76, 25, 23  },
	// This was no match for you!
	{  &res_sprites_start,   8, 14,    0, 100, 14, 23  },
	// Divine powers in action!
	{  &res_sprites_start,  16, 24,    0, 125, 22, 33  },
};
// All these enemies.
static struct Sprite enemies[] = {
	{  &res_enemies_start,    3,  5,    0,   0,  8, 10  },	// Slime
	{  &res_enemies_start,   10,  9,    0,  11, 21, 19  },	// Spider
	{  &res_enemies_start,   13,  4,    0,  31, 31, 13  },	// Rat
	{  &res_enemies_start,   10, 13,    0,  45, 14, 21  },	// Soldier
	{  &res_enemies_start,    5,  6,    0,  67, 11, 11  },	// Robot 1
	{  &res_enemies_start,    9,  9,    0,  79, 18, 15  },	// Robot 2
	{  &res_enemies_start,   10, 13,    0,  95, 15, 22  },	// Guard
	{  &res_enemies_start,   17,  7,    0, 118, 32, 18  },	// Lion
};
// Visual effects of divine abilities!
static struct Sprite visuals[] = {
	// Sword.
	{  &res_visuals_start,    9,  5,    0,   0, 11,  8  },
	{  &res_visuals_start,    9,  5,    0,   9, 11, 11  },
	// Damage.
	{  &res_visuals_start,    2,  2,    0,  21,  5,  4  },
	{  &res_visuals_start,    3,  3,    0,  26,  7,  7  },
	{  &res_visuals_start,    4,  4,    0,  34,  9,  9  },
	// Death.
	{  &res_visuals_start,    7,  3,    0,  44, 14,  7  },
	{  &res_visuals_start,    7,  3,    0,  52, 14,  7  },
	{  &res_visuals_start,    7,  3,    0,  60, 14,  7  },
	{  &res_visuals_start,    7,  3,    0,  68, 14,  7  },
	{  &res_visuals_start,    7,  3,    0,  76, 14,  7  },
	// Ashes (player disappearance).
	{  &res_visuals_start,    3, -5,    0,  84,  7,  4  },
	{  &res_visuals_start,    4,  0,    0,  89,  9,  9  },
	{  &res_visuals_start,    3, -3,    0,  99,  7,  5  },
	// Speed (player disappearing).
	{  &res_visuals_start,    9,  9,    0, 105, 19, 18  },
	{  &res_visuals_start,    9,  9,    0, 124, 19, 18  },
	{  &res_visuals_start,    9,  9,    0, 143, 19, 18  },
	// Thunder.
	{  &res_visuals_start,   14, 23,    0, 162, 28, 30  },
	{  &res_visuals_start,   14, 23,    0, 193, 28, 30  },
	{  &res_visuals_start,   14, 23,    0, 224, 28, 30  },
	{  &res_visuals_start,   14, 23,    0, 255, 28, 30  },
	{  &res_visuals_start,   14, 23,    0, 286, 28, 30  },
	// Fire.
	{  &res_visuals_start,    3,  5,    0, 317,  8, 12  },
	{  &res_visuals_start,    5,  9,    0, 330, 10, 16  },
	{  &res_visuals_start,    6,  14,   0, 347, 12, 21  },
	{  &res_visuals_start,    7,  14,   0, 369, 15, 20  },
	{  &res_visuals_start,    5,  14,   0, 390, 11, 18  },
	// Heal.
	{  &res_visuals_start,    4,  5,    0, 409, 10, 12  },
	{  &res_visuals_start,    7,  6,    0, 422, 16, 16  },
	{  &res_visuals_start,    8,  8,    0, 439, 18, 18  },
	{  &res_visuals_start,   10,  9,    0, 458, 22, 22  },
	{  &res_visuals_start,    8,  7,    0, 481, 18, 18  },
	{  &res_visuals_start,    7,  6,    0, 500, 16, 16  },
	// Special attack.
	{  &res_visuals_start,    9,  8,    0, 518, 17, 11  },
	{  &res_visuals_start,    9,  8,    0, 530, 18, 17  },
	{  &res_visuals_start,    9,  5,    0, 547, 18, 14  },
};



//---
//	Player animation.
//---

// All the NULLs will be initialized by frames_init().
static struct AnimationFrame anim_idle[] = {
	{  &sprites[0], 12,  NULL  },
	{  &sprites[1], 12,  NULL  },
};
static struct AnimationFrame anim_sword[] = {
	{  &sprites[2],  4,  NULL  },
	{  &sprites[3],  4,  NULL  },
};
static struct AnimationFrame anim_victory[] = {
	{  &sprites[5], 48,  NULL  },
};
static struct AnimationFrame anim_dead[] = {
	{  &sprites[4], 48,  NULL  },
};
static struct AnimationFrame anim_skill[] = {
	{  &sprites[6], 24,  NULL  },
	{  &sprites[6], 24,  NULL  },
};

static struct AnimationFrame *anims[] = {
	anim_idle,
	anim_sword,
	anim_victory,
	anim_dead,
	anim_skill,
};



//---
//	Visual effects.
//---

// All the NULLs will be initialized by frames_init().
static struct AnimationFrame visu_damage[] = {
	{  &visuals[2],   2,  NULL  },
	{  &visuals[3],   2,  NULL  },
	{  &visuals[4],   2,  NULL  },
};
static struct AnimationFrame visu_sword[] = {
	{  &visuals[0],   4,  NULL  },
	{  &visuals[1],   4,  NULL  },
};
static struct AnimationFrame visu_death[] = {
	{  &visuals[5],   2,  NULL  },
	{  &visuals[6],   2,  NULL  },
	{  &visuals[7],   2,  NULL  },
	{  &visuals[8],   2,  NULL  },
	{  &visuals[9],   2,  NULL  },
};
static struct AnimationFrame visu_disappear[] = {
	{  &visuals[13],  2,  NULL  },
	{  &visuals[14],  2,  NULL  },
	{  &visuals[15],  2,  NULL  },
};
static struct AnimationFrame visu_thunder[] = {
	{  &visuals[16],  2,  NULL  },
	{  &visuals[17],  2,  NULL  },
	{  &visuals[18],  2,  NULL  },
	{  &visuals[19],  2,  NULL  },
	{  &visuals[20],  2,  NULL  },
};
static struct AnimationFrame visu_fire[] = {
	{  &visuals[21],  2,  NULL  },
	{  &visuals[22],  2,  NULL  },
	{  &visuals[23],  2,  NULL  },
	{  &visuals[24],  2,  NULL  },
	{  &visuals[25],  2,  NULL  },
};
static struct AnimationFrame visu_heal[] = {
	{  &visuals[26],  2,  NULL  },
	{  &visuals[27],  2,  NULL  },
	{  &visuals[28],  2,  NULL  },
	{  &visuals[29],  2,  NULL  },
	{  &visuals[30],  2,  NULL  },
	{  &visuals[31],  2,  NULL  },
};
static struct AnimationFrame visu_special[] = {
	{  &visuals[32],  2,  NULL  },
	{  &visuals[33],  2,  NULL  },
	{  &visuals[34],  2,  NULL  },
};

static struct AnimationFrame *visus[] = {
	visu_damage,
	visu_sword,
	visu_death,
	visu_disappear,
	visu_thunder,
	visu_fire,
	visu_heal,
	visu_special,
};



//---
//	Module API.
//---

/*
	frames_anim()
	Returns the first move of a given animation. The following ones are
	obtained by following the linked list.
*/
struct AnimationFrame *frames_anim(enum Animation anim)
{
	return (anim > Animation_Last) ? (NULL) : (anims[anim]);
}

/*
	frames_visual()
	Returns the first move of a visual effect.
*/
struct AnimationFrame *frames_visual(enum Visual visual)
{
	return (visual > Visual_Last) ? (NULL) : (visus[visual]);
}

/*
	frames_isIdle()
	Checks if the given frame is a frame of an idle animation. This
	function is actually useful: it allows battle animations to have a
	higher animation priority than idle animations.
*/
int frames_isIdle(struct AnimationFrame *frame)
{
	return (!frame || frame == &anim_idle[0] || frame == &anim_idle[1]
		|| frame == &anim_dead[0] || frame == &anim_skill[1]);
}

/*
	frames_enemy()
	Returns the sprite of a given enemy.
*/
struct Sprite *frames_enemy(enum Character enemy)
{
	return (enemy > Character_Max) ? (NULL) : (&enemies[enemy]);
}

/*
	frames_init()
	Initializes the frame data.
*/
void frames_init(void)
{
	// Animation_Idle is a loop between two frames.
	anim_idle[0].next	= &anim_idle[1];
	anim_idle[1].next	= &anim_idle[0];

	// Animation_Sword falls back to Animation_Idle when finished.
	anim_sword[0].next	= &anim_sword[1];
	anim_sword[1].next	= &anim_idle[0];

	// Animation_Victory and Animation_Dead actually end (somehow)!
	anim_victory[0].next	= NULL;
	anim_dead[0].next	= &anim_dead[0];

	// Animation_Skill loops until the called decides when to stop it.
	anim_skill[0].next	= &anim_skill[1];
	anim_skill[1].next	= &anim_skill[1];


	// Visual effects all end.

	visu_sword[0].next	= &visu_sword[1];
	visu_sword[1].next	= NULL;

	visu_damage[0].next	= &visu_damage[1];
	visu_damage[1].next	= &visu_damage[2];
	visu_damage[2].next	= NULL;

	visu_death[0].next	= &visu_death[1];
	visu_death[1].next	= &visu_death[2];
	visu_death[2].next	= &visu_death[3];
	visu_death[3].next	= &visu_death[4];
	visu_death[4].next	= NULL;

	visu_disappear[0].next	= &visu_disappear[1];
	visu_disappear[1].next	= &visu_disappear[2];
	visu_disappear[2].next	= NULL;

	visu_thunder[0].next	= &visu_thunder[1];
	visu_thunder[1].next	= &visu_thunder[2];
	visu_thunder[2].next	= &visu_thunder[3];
	visu_thunder[3].next	= &visu_thunder[4];
	visu_thunder[4].next	= NULL;

	visu_fire[0].next	= &visu_fire[1];
	visu_fire[1].next	= &visu_fire[2];
	visu_fire[2].next	= &visu_fire[3];
	visu_fire[3].next	= &visu_fire[4];
	visu_fire[4].next	= NULL;

	visu_heal[0].next	= &visu_heal[1];
	visu_heal[1].next	= &visu_heal[2];
	visu_heal[2].next	= &visu_heal[3];
	visu_heal[3].next	= &visu_heal[4];
	visu_heal[4].next	= &visu_heal[5];
	visu_heal[5].next	= NULL;

	visu_special[0].next	= &visu_special[1];
	visu_special[1].next	= &visu_special[2];
	visu_special[2].next	= NULL;
}
