#ifndef	_FRAMES_H
#define	_FRAMES_H

#include <display.h>
#include <stddef.h>

/*
	struct Sprite
	Just a combination of an image part and a hot point.
*/
struct Sprite
{
	Image *image;
	// Coordinates of the player/enemy/visual effect center.
	int dx, dy;
	// Part of the image, which represents the sprite.
	int left, top, width, height;
};

/*
	struct AnimationFrame
	An animation frame made of a sprite, a duration (number of timer
	periods after which the frame is finished), and a pointer to the next
	frame.
	This linked list structure is the key of the animation system: it
	allows a one-time animation to link back to the idle animation in a
	second.
*/
struct AnimationFrame
{
	struct Sprite *sprite;
	int duration;	// In units of timer period.
	struct AnimationFrame *next;
};

/*
	enum Animation
	Just a list of all the player animations available in the game.
*/
enum Animation
{
	Animation_Idle		= 0,
	Animation_Sword		= 1,
	Animation_Victory	= 2,
	Animation_Dead		= 3,
	Animation_Skill		= 4,

	Animation_Last		= 4,
};

/*
	enum Visual
	A list of all the visual effects available through the game.
*/
enum Visual
{
	Visual_Damage		= 0,
	Visual_Sword		= 1,
	Visual_Death		= 2,
	Visual_Disappear	= 3,
	Visual_Thunder		= 4,
	Visual_Fire		= 5,
	Visual_Heal		= 6,
	Visual_Special		= 7,

	Visual_Last		= 7,
};

#include "entities.h"

/*
	frames_anim()
	Returns the first move of a given animation. The following ones are
	obtained by following the linked list.
*/
struct AnimationFrame *frames_anim(enum Animation anim);

/*
	frames_visual()
	Returns the first move of a visual effect.
*/
struct AnimationFrame *frames_visual(enum Visual visual);

/*
	frames_isIdle()
	Checks if the given frame is a frame of an idle animation. This
	function is actually useful: it allows battle animations to have a
	higher animation priority than idle animations.
*/
int frames_isIdle(struct AnimationFrame *frame);

/*
	frames_enemy()
	Returns the sprite of a given enemy.
*/
struct Sprite *frames_enemy(enum Character enemy);

/*
	frames_init()
	Initializes the frame data.
*/
void frames_init(void)				__attribute__((constructor));

#endif	// _FRAMES_H
