#include <keyboard.h>
#include <display.h>
#include <string.h>

#include "dungeon.h"
#include "save.h"

#define debug(format, args...) \
	dclear(); \
	dprint(1, 1, format, args); \
	dupdate(); \
	getkey();

//---
//	Main menu.
//---

/*
	level_select()
	Displays a screen where the user may select a level. Returns the level
	number on success, -1 if the user leaves the interface.
*/
int level_select(void)
{
	extern Font res_font_arena_start;
	extern Image res_locked_start;
	extern Image
		res_wall0_start,
		res_wall1_start,
		res_wall2_start;
	Image *bg[] = {
		&res_wall0_start,
		&res_wall1_start,
		&res_wall2_start,
	};
	const char *names[] = {
		"TUTORIAL",
		"PRISON",
		"COLOSSEUM",
	};
	int current = 0, total = 3;
	int i, redraw = 1, x, len;
	int conquered = save()->dungeons_conquered;

	while(1)
	{
		dclear();

		if(current <= conquered) dimage(0, 0, bg[current]);
		else
		{
			dimage(40, 18, &res_locked_start);
			dtext(53, 22, "LOCKED");
		}

		for(i = 0; i < total; i++)
		{
			x = 64 - (11 * total - 9) / 2 + 11 * i;
			dpixel(x, 58, Color_Black);
			dpixel(x + 1, 58, Color_Black);
			dpixel(x, 59, Color_Black);
			dpixel(x + 1, 59, Color_Black);

			if(i != current) continue;

			dline(x - 2, 56, x + 3, 56, Color_Black);
			dline(x - 2, 56, x - 2, 61, Color_Black);
			dline(x + 3, 56, x + 3, 61, Color_Black);
			dline(x - 2, 61, x + 3, 61, Color_Black);
		}

		len = 6 * strlen(names[current]) - 1;
		x = 64 - (len >> 1);

		dclear_area(x - 1, 46, x + len, 52);
		text_configure(&res_font_arena_start, Color_Black);
		dtext(x, 47, names[current]);

		dupdate();
		redraw = 0;

		while(!redraw) switch(getkey_opt(Getkey_NoOption, 0))
		{
		case KEY_SHIFT:
			if(current <= conquered) return current;
			break;
		case KEY_ALPHA:
			return -1;
		case KEY_LEFT:
			if(current) current--, redraw = 1; break;
		case KEY_RIGHT:
			if(current < total - 1) current++, redraw = 1; break;
		}
	}

	// This point should be dead.
	return -1;
}

/*
	main_menu()
	Animates a main menu a displays various options with a moving cursor.
*/
int main_menu(void)
{
	extern Font res_font_arena_start;
	extern Image
		res_menu_background_start,
		res_menu_torches_start,
		res_menu_cursor_start;
	int frame = 0, frame_sub = 0;
	static int selected = 0;

	int cursor = 29 + 9 * selected;
	int cursor_next = cursor;

	text_configure(&res_font_arena_start, Color_Black);

	while(1)
	{
		// Data update.

		if(++frame_sub == 8) frame = (frame + 1) % 4, frame_sub = 0;
		cursor_next = 29 + 9 *selected;
		if(cursor != cursor_next)
			cursor += (cursor < cursor_next) ? (1) : (-1);

		// Drawing.

		dclear();
		dimage(0, 0, &res_menu_background_start);
		dtext(52, 30, "PLAY");
		dtext(43, 39, "CREDITS");
		dtext(52, 48, "QUIT");
		dimage_part(26, 1, &res_menu_torches_start, 8 * frame, 0, 7,
			23);
		dimage_part(94, 1, &res_menu_torches_start,
			8 * ((frame + 2) % 4), 0, 7, 23);
		dimage(17, cursor, &res_menu_cursor_start);
		dupdate();

		// Event handling.

		switch(getkey_opt(Getkey_NoOption, 1))
		{
		case KEY_EXE:
		case KEY_SHIFT:
			return selected;

		case KEY_ALPHA:
		case KEY_MENU:
		case KEY_EXIT:
			return -1;

		case KEY_UP:
			if(selected) selected--;
			break;
		case KEY_DOWN:
			if(selected < 2) selected++;
			break;
		}
	}
}

/*
	credits()
	Displays a minimalist credit screen in the style of the main menu.
*/
void credits(void)
{
	extern Font res_font_arena_start;
	extern Image
		res_menu_background_start,
		res_menu_torches_start,
		res_menu_cursor_start;
	int frame = 0, frame_sub = 0, key, i;

	const char *lines[] = {
		"BY LEPHENIXNOIR",
		"7 DAYS CPC #20",
		"PLANET-CASIO.COM",
		NULL
	};

	text_configure(&res_font_arena_start, Color_Black);

	while(1)
	{
		if(++frame_sub == 8) frame = (frame + 1) % 4, frame_sub = 0;

		dclear();
		dimage(0, 0, &res_menu_background_start);
		for(i = 0; lines[i]; i++)
			dtext(64 - ((6 * strlen(lines[i]) - 1) >> 1),
			30 + 9 * i, lines[i]);
		dimage_part(26, 1, &res_menu_torches_start, 8 * frame, 0, 7,
			23);
		dimage_part(94, 1, &res_menu_torches_start,
			8 * ((frame + 2) % 4), 0, 7, 23);
		dupdate();

		key = getkey_opt(Getkey_NoOption, 1);
		if(key == KEY_ALPHA || key == KEY_MENU || key == KEY_EXIT ||
			key == KEY_SHIFT || key == KEY_EXE) break;
	}
}



//---
//	Main function (execution router).
//---

int main(void)
{
	int x;

	save_load();

	while(1) switch(main_menu())
	{
	case -1:
	case 2:
		save_write();
		return 0;

	case 0:
		x = level_select();
		if(x == -1) break;
		if(dungeon(x)) credits();
		save_write();
		break;

	case 1:
		credits();
		break;
	}

	save_write();
	return 0;
}
