#include "save.h"
#include <string.h>

int Bfile_CreateFile(const unsigned short *filename, int size);
int Bfile_DeleteFile(const unsigned short *filename);
int Bfile_OpenFile(const unsigned short *filename, int mode);
int Bfile_ReadFile(int handle, void *buffer, int size, int whence);
int Bfile_WriteFile(int handle, const void *data, int size);
int Bfile_CloseFile(int handle);

static struct Save save_player;
static const char *carrus_str = "CARRUS";
static const unsigned short filename[] = {
	'\\', '\\', 'f', 'l', 's', '0', '\\',
	'a', 'r', 'e', 'n', 'a', '.', 's', 'a', 'v', 0x00
};

/*
	save()
	Returns the save data address.
*/
struct Save *save(void)
{
	return &save_player;
}

/*
	save_loadFresh()
	Loads a new game into the save data.
*/
static void save_loadFresh(void)
{
	save_player.entity.name		= carrus_str;
	save_player.entity.max_hp	= 80;
	save_player.entity.max_mp	= 12;
	save_player.entity.atk		= 12;
	save_player.entity.def		= 4;
	save_player.entity.spe		= 3;

	save_player.entity.skills[0]	= Skill_None;
	save_player.entity.skills[1]	= Skill_None;
	save_player.entity.skills[2]	= Skill_None;

	save_player.dungeons_conquered	= 0;
}

/*
	save_write()
	Saves to a file.
*/
void save_write(void)
{
	Bfile_DeleteFile(filename);
	Bfile_CreateFile(filename, sizeof(save_player));
	int fd = Bfile_OpenFile(filename, 2);
	Bfile_WriteFile(fd, &save_player, sizeof(save_player));
	Bfile_CloseFile(fd);
}

/*
	save_load()
	Loads from a file. If the file doesn't exist, loads the new game data
	and saves.
*/
int save_load(void)
{
	int fd = Bfile_OpenFile(filename, 1);

	if(fd < 0)
	{
		if(fd != -1) return fd;
		Bfile_CreateFile(filename, sizeof(save_player));

		save_loadFresh();
		save_write();
	}
	else
	{
		Bfile_ReadFile(fd, &save_player, sizeof(save_player), 0);
		Bfile_CloseFile(fd);
	}

	return 0;
}
