//---
//
//	util
//
//	Some functions that may come in handy.
//
//---

#ifndef	_UTIL_H
#define	_UTIL_H

//---
//	references
//	Quite powerful substring handling structure that avoids any dynamic
//	memory allocation.
//---

typedef struct
{
	const char *begin;
	const char *end;
} reference_t;

#define NULLREF ((reference_t){ NULL, NULL })

/*
	refcpy()
	Copies a reference to a string.
*/
char *refcpy(char *str, reference_t ref);



//---
//	Other useful functions.
//---

/*
	sleep()
	Sleeps until an interrupt is triggered.
*/
void sleep(void);

/*
	screen_setContrast()
	Changes the screen contrast setting.
*/
void screen_setContrast(int new_contrast);

/*
	screen_transition()
	Does a screen transition by changing the contrast setting.
*/
void screen_transition(void);

/*
	save_frame()
	Saves the current vram contents to a file. If parameter is non-NULL,
	saves the gray vram in files %%%l.bmp and %%%d.bmp; otherwise saves
	the monochrome vram in file %%%m.bmp.
*/
void save_frame(int gray);

#endif	// _UTIL_H
